"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateFormValue = exports.shouldOmitFieldValue = exports.determineChangedValues = exports.getMissingItems = exports.joinDelimitedValue = exports.splitDelimitedValue = exports.updateFieldValue = exports.registerFields = exports.registerField = exports.getFirstDefinedValue = exports.createField = exports.mapFieldsById = exports.processOptions = exports.processFields = exports.evaluateAllRules = exports.evaluateRule = exports.valuesMatch = exports.fieldDefIsValid = exports.setOptionsInFieldInState = exports.getNextStateFromFields = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _set = require("lodash/set");

var _set2 = _interopRequireDefault(_set);

var _validation = require("../utilities/validation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var getNextStateFromFields = exports.getNextStateFromFields = function getNextStateFromFields(fields, optionsHandler, validationHandler, parentContext) {
  fields = processFields(fields);
  if (optionsHandler) {
    fields = processOptions(fields, optionsHandler, parentContext);
  }
  fields = (0, _validation.validateAllFields)(fields, validationHandler, parentContext);
  var value = calculateFormValue(fields);
  var isValid = fields.every(function (field) {
    return field.isValid;
  });
  var nextState = {
    fields: fields,
    value: value,
    isValid: isValid
  };
  return nextState;
};

var setOptionsInFieldInState = exports.setOptionsInFieldInState = function setOptionsInFieldInState(prevState, field, options) {
  var fieldIndex = prevState.fields.findIndex(function (prevField) {
    return prevField.id === field.id;
  });

  field.options = options;
  field.pendingOptions = undefined;

  var prevFields = prevState.fields;

  return {
    fields: [].concat(_toConsumableArray(prevFields.slice(0, fieldIndex)), [field], _toConsumableArray(prevFields.slice(fieldIndex + 1)))
  };
};

// A field definition is valid if a field with the same id does not already exist in
// the supplied form state.
// We are assuming that typing takes care that all required attributes are present
var fieldDefIsValid = exports.fieldDefIsValid = function fieldDefIsValid(field, fields) {
  return !fields.some(function (currentField) {
    return currentField.id === field.id;
  });
};

var valuesMatch = exports.valuesMatch = function valuesMatch(a, b) {
  if (a && b) {
    return a.toString() === b.toString();
  } else {
    return a === b;
  }
};

var evaluateRule = exports.evaluateRule = function evaluateRule() {
  var rule = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var targetValue = arguments[1];
  var _rule$is = rule.is,
      is = _rule$is === undefined ? [] : _rule$is,
      _rule$isNot = rule.isNot,
      isNot = _rule$isNot === undefined ? [] : _rule$isNot;

  var hasValidValue = is.length === 0;
  var hasInvalidValue = !!rule.isNot && rule.isNot.length > 0;

  if (hasInvalidValue) {
    hasInvalidValue = isNot.some(function (invalidValue) {
      if (invalidValue.hasOwnProperty("value")) {
        return valuesMatch(invalidValue.value, targetValue);
      } else {
        return valuesMatch(invalidValue, targetValue);
      }
    });
  }

  if (!hasInvalidValue && !hasValidValue) {
    if (rule.is && rule.is.length) {
      hasValidValue = rule.is.some(function (validValue) {
        if (validValue.hasOwnProperty("value")) {
          return valuesMatch(validValue.value, targetValue);
        } else {
          return valuesMatch(validValue, targetValue);
        }
      });
    }
  }
  return hasValidValue && !hasInvalidValue;
};

var evaluateAllRules = exports.evaluateAllRules = function evaluateAllRules() {
  var rules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var fieldsById = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var defaultResult = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

  var rulesPass = defaultResult;
  if (rules.length) {
    rulesPass = rules.some(function (rule) {
      if (rule.field && fieldsById.hasOwnProperty(rule.field)) {
        return evaluateRule(rule, fieldsById[rule.field].value);
      } else {
        return defaultResult;
      }
    });
  }
  return rulesPass;
};

var processFields = exports.processFields = function processFields(fields) {
  var fieldsById = mapFieldsById(fields);
  var updatedFields = fields.map(function (field) {
    var defaultValue = field.defaultValue,
        value = field.value,
        visible = field.visible,
        required = field.required,
        disabled = field.disabled,
        trimValue = field.trimValue,
        _field$visibleWhen = field.visibleWhen,
        visibleWhen = _field$visibleWhen === undefined ? [] : _field$visibleWhen,
        _field$requiredWhen = field.requiredWhen,
        requiredWhen = _field$requiredWhen === undefined ? [] : _field$requiredWhen,
        _field$disabledWhen = field.disabledWhen,
        disabledWhen = _field$disabledWhen === undefined ? [] : _field$disabledWhen;


    var processedValue = typeof value !== "undefined" ? value : defaultValue;
    if (trimValue && processedValue && typeof processedValue.trim === "function") {
      processedValue = processedValue.trim();
    }

    return _extends({}, field, {
      value: processedValue,
      visible: evaluateAllRules(visibleWhen, fieldsById, visible !== false),
      required: evaluateAllRules(requiredWhen, fieldsById, !!required),
      disabled: evaluateAllRules(disabledWhen, fieldsById, !!disabled)
    });
  });
  return updatedFields;
};

var processOptions = exports.processOptions = function processOptions(fields, optionsHandler, parentContext) {
  return fields.map(function (field) {
    var id = field.id,
        options = field.options;

    if (!options) {
      var handlerOptions = optionsHandler(id, fields, parentContext);
      if (handlerOptions instanceof Promise) {
        field.options = [];
        field.pendingOptions = handlerOptions;
      } else if (handlerOptions) {
        field.options = handlerOptions;
        field.pendingOptions = undefined;
      }
    }
    return field;
  });
};

var mapFieldsById = exports.mapFieldsById = function mapFieldsById(fields) {
  return fields.reduce(function (map, field) {
    map[field.id] = field;
    return map;
  }, {});
};

var createField = exports.createField = function createField(field) {
  var _field$id = field.id,
      id = _field$id === undefined ? "" : _field$id,
      _field$name = field.name,
      name = _field$name === undefined ? "" : _field$name,
      _field$type = field.type,
      type = _field$type === undefined ? "" : _field$type,
      _field$placeholder = field.placeholder,
      placeholder = _field$placeholder === undefined ? "" : _field$placeholder,
      _field$value = field.value,
      value = _field$value === undefined ? undefined : _field$value,
      _field$visible = field.visible,
      visible = _field$visible === undefined ? true : _field$visible,
      _field$required = field.required,
      required = _field$required === undefined ? false : _field$required,
      _field$disabled = field.disabled,
      disabled = _field$disabled === undefined ? false : _field$disabled,
      _field$visibleWhen2 = field.visibleWhen,
      visibleWhen = _field$visibleWhen2 === undefined ? [] : _field$visibleWhen2,
      _field$requiredWhen2 = field.requiredWhen,
      requiredWhen = _field$requiredWhen2 === undefined ? [] : _field$requiredWhen2,
      _field$disabledWhen2 = field.disabledWhen,
      disabledWhen = _field$disabledWhen2 === undefined ? [] : _field$disabledWhen2,
      _field$validWhen = field.validWhen,
      validWhen = _field$validWhen === undefined ? {} : _field$validWhen,
      _field$isValid = field.isValid,
      isValid = _field$isValid === undefined ? true : _field$isValid,
      _field$errorMessages = field.errorMessages,
      errorMessages = _field$errorMessages === undefined ? "" : _field$errorMessages;

  return {
    id: id,
    name: name,
    type: type,
    placeholder: placeholder,
    value: value,
    visible: visible,
    required: required,
    disabled: disabled,
    visibleWhen: visibleWhen,
    requiredWhen: requiredWhen,
    disabledWhen: disabledWhen,
    isValid: isValid,
    validWhen: validWhen,
    errorMessages: errorMessages
  };
};

var getFirstDefinedValue = exports.getFirstDefinedValue = function getFirstDefinedValue() {
  for (var _len = arguments.length, values = Array(_len), _key = 0; _key < _len; _key++) {
    values[_key] = arguments[_key];
  }

  var valueToReturn = void 0;
  values.some(function (value) {
    if (typeof value !== "undefined") {
      valueToReturn = value;
      return true;
    }
    return false;
  });
  return valueToReturn;
};

// Because this function can be passed with the state of a component form
// it is not mutating the supplied fields array but returning a new instance
// each time, this is less efficient (when passing entire fieldDef arrays to the
// form) but safer when children of forms are registering themselves
var registerField = exports.registerField = function registerField(field, fields, formValue) {
  if (fieldDefIsValid(field, fields)) {
    var defaultValue = field.defaultValue,
        name = field.name,
        value = field.value,
        valueDelimiter = field.valueDelimiter;

    field.defaultValue = getFirstDefinedValue(formValue[name], value, defaultValue);
    field.value = splitDelimitedValue(value, valueDelimiter);
    return fields.concat(field);
  }
  return fields.slice();
};

var registerFields = exports.registerFields = function registerFields(fieldsToValidate, formValue) {
  var fields = [];
  fieldsToValidate.forEach(function (field) {
    if (fieldDefIsValid(field, fields)) {
      var defaultValue = field.defaultValue,
          name = field.name,
          value = field.value,
          valueDelimiter = field.valueDelimiter;

      var initialValue = getFirstDefinedValue(formValue[name], value, defaultValue);
      field.defaultValue = initialValue;
      field.value = splitDelimitedValue(initialValue, valueDelimiter);
      fields.push(field);
    }
  });
  return fields;
};

var updateFieldValue = exports.updateFieldValue = function updateFieldValue(id, value, fields) {
  var fieldsById = mapFieldsById(fields);
  var updateValue = typeof value !== "undefined" && value;
  var field = fieldsById[id];
  if (field.omitWhenHidden && !field.visible) {
    console.log("Not updating field value for", field);
  } else {
    field.value = updateValue;
  }
  return fields;
};

var splitDelimitedValue = exports.splitDelimitedValue = function splitDelimitedValue(value, valueDelimiter) {
  if (valueDelimiter) {
    if (typeof value === "string") {
      value = value.split(valueDelimiter);
    } else {
      value = [];
    }
  }
  return value;
};

var joinDelimitedValue = exports.joinDelimitedValue = function joinDelimitedValue(value, valueDelimiter) {
  if (Array.isArray(value) && valueDelimiter) {
    value = value.join(valueDelimiter);
  }
  return value;
};

var getMissingItems = exports.getMissingItems = function getMissingItems(missingFrom, foundIn) {
  return foundIn.reduce(function (missingItems, item) {
    !missingFrom.includes(item) && missingItems.push(item);
    return missingItems;
  }, []);
};

var determineChangedValues = exports.determineChangedValues = function determineChangedValues(field) {
  var name = field.name,
      defaultValue = field.defaultValue,
      value = field.value,
      valueDelimiter = field.valueDelimiter,
      _field$addedSuffix = field.addedSuffix,
      addedSuffix = _field$addedSuffix === undefined ? "_added" : _field$addedSuffix,
      _field$removedSuffix = field.removedSuffix,
      removedSuffix = _field$removedSuffix === undefined ? "_removed" : _field$removedSuffix;

  var outputValues = [];

  var initialValue = splitDelimitedValue(defaultValue, valueDelimiter);
  if (Array.isArray(initialValue) && Array.isArray(value)) {
    var added = getMissingItems(initialValue, value);
    var removed = getMissingItems(value, initialValue);

    added = joinDelimitedValue(added, valueDelimiter);
    removed = joinDelimitedValue(removed, valueDelimiter);

    outputValues.push({
      name: name + (addedSuffix || "_added"),
      value: added
    }, {
      name: name + (removedSuffix || "_removed"),
      value: removed
    });
  }
  return outputValues;
};

var shouldOmitFieldValue = exports.shouldOmitFieldValue = function shouldOmitFieldValue(field) {
  var omitWhenHidden = field.omitWhenHidden,
      _field$omitWhenValueI = field.omitWhenValueIs,
      omitWhenValueIs = _field$omitWhenValueI === undefined ? [] : _field$omitWhenValueI,
      visible = field.visible,
      value = field.value;

  return omitWhenHidden && !visible || omitWhenValueIs.length !== 0 && omitWhenValueIs.some(function (currValue) {
    return value === currValue;
  });
};

var calculateFormValue = exports.calculateFormValue = function calculateFormValue(fields) {
  return fields.reduce(function (formValue, field) {
    var name = field.name,
        value = field.value,
        useChangesAsValues = field.useChangesAsValues;

    if (shouldOmitFieldValue(field)) {
      return formValue;
    } else if (useChangesAsValues) {
      determineChangedValues(field).forEach(function (_ref) {
        var name = _ref.name,
            value = _ref.value;
        return (0, _set2.default)(formValue, name, value);
      });
    } else {
      (0, _set2.default)(formValue, name, value);
    }

    return formValue;
  }, {});
};