"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _utils = require("./utils");

var field1 = (0, _utils.createField)({
  id: "one",
  name: "name",
  value: "value"
});

describe("registerFields", function () {
  var field1 = (0, _utils.createField)({
    id: "a",
    name: "a",
    type: "text"
  });
  var field2 = (0, _utils.createField)({
    id: "b",
    name: "b",
    type: "text"
  });
  var field3 = (0, _utils.createField)({
    id: "a",
    name: "c",
    type: "text"
  });

  var fields = [field1, field2, field3];

  test("fields with duplicate IDs are filtered out", function () {
    var registeredFields = (0, _utils.registerFields)(fields, {});
    expect(registeredFields.length).toEqual(2);
    expect(registeredFields[0].id).toEqual("a");
    expect(registeredFields[1].id).toEqual("b");
  });
});

describe("fieldDefIsValid", function () {
  it("field is valid when the form does not contain a field with the same id", function () {
    expect((0, _utils.fieldDefIsValid)(field1, [])).toEqual(true);
  });

  test("field is not valid when form already contains a field with the same id", function () {
    expect((0, _utils.fieldDefIsValid)(field1, [field1])).toEqual(false);
  });
});

describe("evaluateRule", function () {
  test("evaluting a rule with no arguments", function () {
    expect((0, _utils.evaluateRule)()).toEqual(true);
  });

  test("successful 'is' rule", function () {
    expect((0, _utils.evaluateRule)({
      field: "one",
      rule: {
        is: [true]
      },
      targetValue: true
    })).toEqual(true);
  });

  test("failing 'is' rule", function () {
    expect((0, _utils.evaluateRule)({
      field: "one",
      is: [false]
    }, true)).toEqual(false);
  });

  test("successful 'isNot' rule", function () {
    expect((0, _utils.evaluateRule)({
      field: "one",
      isNot: [true]
    }, false)).toEqual(true);
  });

  test("failing 'isNot' rule", function () {
    expect((0, _utils.evaluateRule)({
      field: "one",
      isNot: [false]
    }, false)).toEqual(false);
  });

  test("successful combined 'is' and isNot' rule", function () {
    expect((0, _utils.evaluateRule)({
      field: "onee",
      is: [true],
      isNot: [false]
    }, true)).toEqual(true);
  });

  test("failing combined 'is' and isNot' rule", function () {
    expect((0, _utils.evaluateRule)({
      field: "one",
      is: [true],
      isNot: [false]
    }, false)).toEqual(false);
  });

  // NOTE: This is one option for allowing form builder to construct rules, but is harder to implement
  test("works with complex objects", function () {
    expect((0, _utils.evaluateRule)({
      field: "one",
      is: [{ value: "bob" }],
      isNot: [{ value: "ted" }]
    }, "bob")).toEqual(true);
  });
});

// describe("evaluateAllRules", () => {
//   test("evaluating no rules returns the default default", () => {
//     expect(evaluateAllRules()).toEqual(true);
//   });
// });

describe("processFields", function () {
  var triggerField = (0, _utils.createField)({
    id: "triggerField",
    name: "triggerField",
    value: "test"
  });
  var shouldBeVisible = (0, _utils.createField)({
    id: "shouldBeVisible",
    name: "shouldBeVisible",
    visibleWhen: [{
      field: "triggerField",
      is: ["test"]
    }]
  });
  var shouldBeHidden = (0, _utils.createField)({
    id: "shouldBeHidden",
    name: "shouldBeHidden",
    visibleWhen: [{
      field: "triggerField",
      isNot: ["test"]
    }]
  });
  var shouldBeRequired = (0, _utils.createField)({
    id: "shouldBeRequired",
    name: "shouldBeRequired",
    requiredWhen: [{
      field: "triggerField",
      is: ["test"]
    }]
  });
  var shouldBeOptional = (0, _utils.createField)({
    id: "shouldBeOptional",
    name: "shouldBeOptional",
    requiredWhen: [{
      field: "triggerField",
      isNot: ["test"]
    }]
  });
  var shouldBeDisabled = (0, _utils.createField)({
    id: "shouldBeDisabled",
    name: "shouldBeDisabled",
    disabledWhen: [{
      field: "triggerField",
      is: ["test"]
    }]
  });
  var shouldBeEnabled = (0, _utils.createField)({
    id: "shouldBeEnabled",
    name: "shouldBeEnabled",
    disabledWhen: [{
      field: "triggerField",
      isNot: ["test"]
    }]
  });

  var fields = [triggerField, shouldBeVisible, shouldBeHidden, shouldBeRequired, shouldBeOptional, shouldBeDisabled, shouldBeEnabled];

  var processedFields = (0, _utils.processFields)(fields);
  var processedFieldsById = (0, _utils.mapFieldsById)(processedFields);

  test("field should be visible", function () {
    expect(processedFieldsById.shouldBeVisible.visible).toBe(true);
  });
  test("field should be hidden", function () {
    expect(processedFieldsById.shouldBeHidden.visible).toBe(false);
  });
  test("field should be required", function () {
    expect(processedFieldsById.shouldBeRequired.required).toBe(true);
  });
  test("field should be optional", function () {
    expect(processedFieldsById.shouldBeOptional.required).toBe(false);
  });
  test("field should be disabled", function () {
    expect(processedFieldsById.shouldBeDisabled.disabled).toBe(true);
  });
  test("field should be enabled", function () {
    expect(processedFieldsById.shouldBeEnabled.disabled).toBe(false);
  });
});

describe("shouldOmitFieldValue", function () {
  var baseField = {
    id: "TEST",
    value: "foo",
    name: "test",
    type: "text"
  };
  test("value should be omitted when hidden", function () {
    var field = _extends({}, baseField, {
      omitWhenHidden: true,
      visible: false
    });
    expect((0, _utils.shouldOmitFieldValue)(field)).toEqual(true);
  });
  test("value should be included when visible", function () {
    var field = _extends({}, baseField, {
      omitWhenHidden: true,
      visible: true
    });
    expect((0, _utils.shouldOmitFieldValue)(field)).toEqual(false);
  });
  test("value should be ommitted when value matches", function () {
    var field = _extends({}, baseField, {
      omitWhenValueIs: ["foo"]
    });
    expect((0, _utils.shouldOmitFieldValue)(field)).toEqual(true);
  });
  test("value should be included when value does not match", function () {
    var field = _extends({}, baseField, {
      omitWhenValueIs: ["wrong"]
    });
    expect((0, _utils.shouldOmitFieldValue)(field)).toEqual(false);
  });
});

describe("calculateFormValue", function () {
  var baseField = {
    id: "TEST",
    value: "foo",
    name: "test",
    type: "text"
  };
  var field1 = _extends({}, baseField, {
    omitWhenHidden: true,
    visible: false
  });
  var field2 = _extends({}, baseField, {
    id: "TEST2",
    name: "test2",
    value: "bar"
  });
  var field3 = _extends({}, baseField, {
    id: "TEST3",
    name: "test3",
    value: "bob"
  });
  var field4 = _extends({}, baseField, {
    id: "TEST4",
    name: "test3",
    value: "ted"
  });
  var field5 = _extends({}, baseField, {
    id: "TEST5",
    name: "test.dot.notation",
    value: "ted"
  });

  var value = (0, _utils.calculateFormValue)([field1, field2, field3, field4]);
  test("two field values should be omitted", function () {
    expect(Object.keys(value).length).toEqual(2);
  });
  test("hidden field value should not be included", function () {
    expect(value.test).not.toBeDefined();
  });
  test("normal value should be included", function () {
    expect(value.test2).toEqual("bar");
  });
  test("last field wins", function () {
    expect(value.test3).toEqual("ted");
  });

  test("dot-notation names can be provided", function () {
    var value = (0, _utils.calculateFormValue)([field5]);
    expect(value.test.dot.notation).toEqual("ted");
  });

  test("get added and removed values", function () {
    var field1 = _extends({}, baseField, {
      defaultValue: "1,2,3",
      value: ["2", "4", "5"], // HERE BE DRAGONS - will the new value really always be an array?
      valueDelimiter: ",",
      useChangesAsValues: true
    });
    var value = (0, _utils.calculateFormValue)([field1]);
    expect(value.test_added).toEqual("4,5");
    expect(value.test_removed).toEqual("1,3");
  });

  // test("dot-notation values setting", () => {
  //   const field1 = {
  //     name: "some.nested.prop",
  //     ...baseField,
  //     value: "foo"
  //   };
  //   const value = calculateFormValue([field1]);
  //   expect(value.some.nested.prop).toEqual("foo");
  // });
});

describe("updateFieldValue", function () {
  var field1 = {
    id: "A",
    name: "a",
    type: "text",
    value: "baa"
  };
  var field2 = {
    id: "B",
    name: "b",
    type: "text",
    value: "moo"
  };
  var field3 = {
    id: "C",
    name: "c",
    type: "text",
    value: "woof"
  };

  var fields = (0, _utils.updateFieldValue)("B", "oink", [field1, field2, field3]);
  var fieldsById = (0, _utils.mapFieldsById)(fields);

  test("field is updated with new value", function () {
    expect(fieldsById.B.value).toEqual("oink");
  });
});

describe("joinDelimitedValue", function () {
  test("join with commas", function () {
    expect((0, _utils.joinDelimitedValue)([1, 2, 3], ",")).toEqual("1,2,3");
  });

  test("leave non-array values as-id", function () {
    expect((0, _utils.joinDelimitedValue)("test", ",")).toEqual("test");
  });
});

describe("splitDelimitedValue", function () {
  test("split on commas", function () {
    // NOTE: Always becomes an array of strings
    //       Is this worth parsing?
    expect((0, _utils.splitDelimitedValue)("1,2,3", ",")).toEqual(["1", "2", "3"]);
  });

  test("create an array from an non-delimited value", function () {
    expect((0, _utils.splitDelimitedValue)("test", ",")).toEqual(["test"]);
  });

  test("leave value as is if no delimiter provided", function () {
    expect((0, _utils.splitDelimitedValue)("test")).toEqual("test");
  });
});

describe("getMissingItems", function () {
  test("a is missing from [b,c] but is in [a,b,c]", function () {
    expect((0, _utils.getMissingItems)(["b", "c"], ["a", "b", "c"])).toEqual(["a"]);
  });
});

describe("determineChangedValues", function () {
  var field = {
    id: "TEST",
    name: "foo",
    type: "text",
    value: ["a", "c", "e", "f"],
    defaultValue: ["a", "b", "c", "d"]
  };

  var changes = (0, _utils.determineChangedValues)(field);
  test("output structure is correct", function () {
    expect(changes.length).toEqual(2);
    expect(changes[0].name).toEqual("foo_added");
    expect(changes[1].name).toEqual("foo_removed");
  });
  test("e and f were added", function () {
    expect(changes[0].value).toEqual(["e", "f"]);
  });
  test("b and d were removed", function () {
    expect(changes[1].value).toEqual(["b", "d"]);
  });
});

describe("getFirstDefinedValue", function () {
  test("boolean", function () {
    expect((0, _utils.getFirstDefinedValue)(undefined, undefined, false, true)).toEqual(false);
  });
  test("number", function () {
    expect((0, _utils.getFirstDefinedValue)(undefined, 0, 10)).toEqual(0);
  });
});

describe("default value handling", function () {
  var field = {
    id: "WITH_DEFAULT",
    name: "test",
    type: "text",
    defaultValue: "bob"
  };

  test("default value is assigned to value", function () {
    var processedFields = (0, _utils.processFields)([field]);
    expect(processedFields[0].value).toEqual("bob");
  });

  test("Value takes precedence over defaultValue", function () {
    field.value = "ted";
    var processedFields = (0, _utils.processFields)([field]);
    expect(processedFields[0].value).toEqual("ted");
  });

  test("Falsy value takes precedence over defaultValue", function () {
    field.value = false;
    var processedFields = (0, _utils.processFields)([field]);
    expect(processedFields[0].value).toEqual(false);
  });
});

describe("trimming behaviour", function () {
  var field = {
    id: "TO_BE_TRIMMED",
    name: "test",
    type: "text",
    value: "   foo     ",
    trimValue: true
  };

  test("leading and trailing whitespace is removed from value", function () {
    var processedFields = (0, _utils.processFields)([field]);
    var trimmedField = processedFields[0];
    expect(trimmedField.value).toEqual("foo");
  });
});

describe("setOptionsInFieldInState", function () {
  var field1 = (0, _utils.createField)({
    id: "a",
    name: "a",
    type: "text"
  });
  var field2 = (0, _utils.createField)({
    id: "b",
    name: "b",
    type: "text"
  });
  var field3 = (0, _utils.createField)({
    id: "c",
    name: "c",
    type: "text"
  });

  var fields = [field1, field2, field3];
  var state = {
    fields: fields,
    value: {},
    isValid: true,
    defaultFields: []
  };

  var options = [{
    items: ["one", "two", "three"]
  }];

  var updatedState = (0, _utils.setOptionsInFieldInState)(state, field2, options);

  test("leaves fields in state with the correct length", function () {
    expect(updatedState.fields).toHaveLength(3);
  });

  test("leaves fields in state in the same order", function () {
    expect(updatedState.fields[0].id).toBe("a");
    expect(updatedState.fields[1].id).toBe("b");
    expect(updatedState.fields[2].id).toBe("c");
  });

  test("assigns options to teh correct field", function () {
    expect(updatedState.fields[0].options).toBeUndefined();
    expect(updatedState.fields[1].options).toBe(options);
    expect(updatedState.fields[2].options).toBeUndefined();
  });

  test("assigns pending options as undefined", function () {
    expect(updatedState.fields[1].pendingOptions).toBeUndefined();
  });
});